/* 
 *   ohlpcustomuifunctions.js
 *   This file contains functions that updates ui behaviors in the OHLP Portal website.
 *   
 *   !important! in order to upload the updated script, please make a copy of the updated file and remove the ".js" file extension. 
 *    the file with js extension can't be imported in Power pgaes Management App.
 * 
 *   Contents of this file:
 *   |no-scroll-in-grid|
 *   |loading-overlay|
 *   |retain-tab-focus|
*/

const localStorgeKeyForBusinessEntity = "activeTab-BusinessEntity";
const localStorgeKeyForAccountLocation = "activeTab-AccountLocation";
const localStorgeKeyForTaxingDistrict = "activeTab-TaxingDistrict";

// *loading-overlay*
function showLoading(message = "Loading, please wait...") {
    const overlay = document.getElementById("loadingOverlay");
    const messageElement = document.getElementById("loadingMessage");
    if (messageElement) {
        messageElement.textContent = message;
    }
    if (overlay) {
        overlay.style.display = "flex";
    }
}

function hideLoading() {
    const overlay = document.getElementById("loadingOverlay");
    if (overlay) {
        overlay.style.display = "none";
    }
}

function clearCurrentTabAtBusinessEntity() {
    localStorage.removeItem(localStorgeKeyForBusinessEntity);
}
function clearCurrentTabAtAccountLocation() {
    localStorage.removeItem(localStorgeKeyForAccountLocation);
}
function clearCurrentTabAtTaxingDistrict() {
    localStorage.removeItem(localStorgeKeyForTaxingDistrict);
}

function convertRadioGroupToVertical(elementId) {
    const $radioButtons = $('#' + elementId + ' input[type="radio"]');

    $radioButtons.each(function () {
        // Create a new div element
        const $div = $('<div></div>');

        // Get the parent node of the radio button
        const $parent = $(this).parent();

        // Get the next sibling of the radio button
        const $nextSibling = $(this).next();

        // Append the radio button and the next sibling to the new div
        $div.append($(this));
        $div.append($nextSibling);

        // Insert both the radio and the sibiling to the new div
        $parent.append($div);
    });
}

//Make mandatory field
function AddRequired(fieldName, isLabelPresent = true, customerrormessage = "") {
    //console.log("AddRequired is called for: " + fieldName);
    try {
        let validators = Page_Validators.filter(function (el) {
            return el.id == fieldName + "Validator";
        }
        );
        if (validators == undefined || validators == null || validators.length == 0) {
            if ($("#" + fieldName) != undefined) {
                $("#" + fieldName).prop('required', true);
                $("#" + fieldName).closest(".control").prev().addClass("required");
                var Requiredvalidator = document.createElement('span');


                // Create new validator
                var focusField = "javascript:scrollToAndFocus('" + fieldName + "_label','" + fieldName + "');return&nbsp;false;";
                var errorMessage = "";
                var fieldLabelId = "#" + fieldName + "_label";
                if (isLabelPresent) {
                    errorMessage = $("#" + fieldName + "_label").html() + " is a required field.";
                    //console.log("isLabelPresent is true");
                }
                else {
                    fieldLabelId = "#" + fieldName;
                    errorMessage = fieldName + " is a required field.";
                    focusField = "javascript:scrollToAndFocus('" + fieldName + "','" + fieldName + "');return&nbsp;false;";
                    //console.log("isLabelPresent is false");
                }
                if (customerrormessage != "") {
                    errorMessage = customerrormessage;
                    //console.log("Custom error message is not null");
                }

                Requiredvalidator.style.display = "none";
                Requiredvalidator.id = fieldName + "Validator";
                Requiredvalidator.controltovalidate = fieldName;
                Requiredvalidator.errormessage = "<a href='" + fieldLabelId + "' onclick=" + focusField + " referencecontrolid=" + fieldName + ">" + errorMessage + "</a>";
                Requiredvalidator.initialvalue = "";

                if ($("#" + fieldName).is('input[type="file"]')) {
                    Requiredvalidator.evaluationfunction = function () {
                        var fileInput = $("#" + fieldName)[0]; // Get the DOM element
                        if (fileInput.files.length > 0) {
                            return true;
                        }
                        else {
                            return false;
                        }
                    };
                } else if($("#" + fieldName).find('input[type="radio"]').length>0)  {

                    Requiredvalidator.evaluationfunction = function () {
                        var value = $("#" + fieldName).closest('.boolean-radio').find(':checked').val();
                        if (value == null || value == "") {
                            return false;
                        } else {
                            return true;
                        }
                    };
                }else {

                    Requiredvalidator.evaluationfunction = function () {
                        var value = $("#" + fieldName).val();
                        if (value == null || value == "") {
                            return false;
                        } else {
                            return true;
                        }
                    };
                }


                // Add the new validator to the page validators array:

                Page_Validators.push(Requiredvalidator);
            }
        }

    }
    catch (error) {
        console.log("Error at AddRequired: " + fieldName + ":" + error);
    }
}
//Make UnMandatory
function RemoveRequired(fieldName) {
    //console.log("RemoveRequired is called for: " + fieldName);
    try {
        if ($("#" + fieldName) != undefined) {
            $("#" + fieldName).closest(".control").prev().removeClass("required");
            $("#" + fieldName).prop('required', false);

            for (i = 0; i < Page_Validators.length; i++) {
                if (Page_Validators[i].id == fieldName + "Validator") {
                    //console.log("Removing: " + Page_Validators[i].id);
                    Page_Validators.splice(i,1);
                    //console.log("Current validator count is: " + Page_Validators.length + ".");
                }
            }
        }
    }
    catch (error) {
        console.log("Error at RemoveRequired: " + fieldName + ":" + error);
    }
}

//Add a validation to check if the value of booth fields match
function AddTwoFieldsMatchValidation(fieldName1, fieldName2, customerrormessage = "") {
    try {
        let validators = Page_Validators.filter(function (el) {
            return el.id == fieldName2 + "MatchValidator";
        });
        
        if(validators != undefined && validators != null && validators.length > 0)
        {
            for (i = 0; i < Page_Validators.length; i++) {
                if (Page_Validators[i].id == fieldName2 + "MatchValidator") {
                    Page_Validators.splice(i);
                }
            }
        }

        if ($("#" + fieldName1) != undefined && $("#" + fieldName2) != undefined) {
            var FieldsMatchValidator = document.createElement('span');

            // Create new validator
            var focusField="javascript:scrollToAndFocus('" + fieldName2 + "_label','" + fieldName2 + "');return&nbsp;false;"; 
            var errorMessage = "";
            var fieldLabelId = "#" + fieldName2 + "_label";
            errorMessage = "Values on " + $("#" + fieldName1 + "_label").html() + " and " + $("#" + fieldName2 + "_label").html() + " must be present and the same.";

            if (customerrormessage != "") {
                errorMessage = customerrormessage;
                //console.log("Custom error message is not null");
            }        
    
            FieldsMatchValidator.style.display = "none";
            FieldsMatchValidator.id = fieldName2 + "MatchValidator";
            FieldsMatchValidator.controltovalidate = fieldName2;
            FieldsMatchValidator.errormessage = "<a href='" + fieldLabelId + "' onclick=" + focusField + " referencecontrolid=" + fieldName2 + ">" + errorMessage + "</a>"; 
            FieldsMatchValidator.initialvalue = "";

            FieldsMatchValidator.evaluationfunction = function () {
                var value1 = $("#" + fieldName1).val();
                var value2 = $("#" + fieldName2).val();
                if (value1 != null && value1 != "" && value2 != null && value2 != "" && value1 == value2) {
                    return true;
                } else {
                    return false;
                }
            };

            // Add the new validator to the page validators array:
            Page_Validators.push(FieldsMatchValidator);
        }
    } catch (error) {
        console.log("Error at FieldsMatch: " + fieldName2 + ": " + error);
    }
}

function SetDefaultValueToLookup(fieldSchemaName, guid, name, tableSchemaName) {
    try {
        var lookupField = $("#" + fieldSchemaName);
        if (lookupField) {
            $("#" + fieldSchemaName + "_name").val(name);
            $("#" + fieldSchemaName).val(guid);
            $("#" + fieldSchemaName + "_entityname").val(tableSchemaName);
        }
    }
    catch (error) {
        console.log("Error at SetLookupDefaultValue: " + fieldSchemaName);
    }
}

$(document).ready(function () {
    const queryString = window.location.search;
    const urlParams = new URLSearchParams(queryString);
    // *no-scroll-in-grid*
    // remove scroll bar when there is 1-2 rows in a grid and an action bar appears.
    // removing scroll bar by dynamically changing row height when an action bar pops up.
    $(".entity-grid").on("loaded", function () {
        $(this).children(".view-grid").find("div.dropdown.action").each(function (i, e) {
            var trHeight = $(this).closest("tr").height();
            $(this).on('shown.bs.dropdown', function () {
                $(this).closest("tr").height($(this).height() * 2 + $(this).find("ul.dropdown-menu.show").height());
            });
            $(this).on('hidden.bs.dropdown', function () {
                $(this).closest("tr").height(trHeight);
            });
        });
    });

    // *retain-tab-focus*
    // remove scroll bar when there is 1-2 rows in a grid and an action bar appears.
    // removing scroll bar by dynamically changing row height when an action bar pops up.


    $('#tabs-business-entity button[data-bs-toggle="tab"]').on('click', function (e) {
        try {
            localStorage.setItem(localStorgeKeyForBusinessEntity, e.target.dataset.bsTarget);
        } catch (e) {
            console.log(e);
        }
    });

    $('#tabs-account-location button[data-bs-toggle="tab"]').on('click', function (e) {
        try {
            localStorage.setItem(localStorgeKeyForAccountLocation, e.target.dataset.bsTarget);
        } catch (e) {
            console.log(e);
        }
    });
    $('#tabs-taxing-district button[data-bs-toggle="tab"]').on('click', function (e) {
        try {
            localStorage.setItem(localStorgeKeyForTaxingDistrict, e.target.dataset.bsTarget);
        } catch (e) {
            console.log(e);
        }
    });


    const tabId = urlParams.get('tabId');

    if (tabId) {
        if ($("#tabs-business-entity").length > 0) {
            localStorage.setItem(localStorgeKeyForBusinessEntity, "#" + tabId);
        }

        if ($("#tabs-account-location").length > 0) {
            localStorage.setItem(localStorgeKeyForAccountLocation, "#" + tabId);
        }

        if ($("#tabs-taxing-district").length > 0) {
            localStorage.setItem(localStorgeKeyForTaxingDistrict, "#" + tabId);
        }
    }

    try {
        var activeTabEntity = localStorage.getItem(localStorgeKeyForBusinessEntity);
        var activeTabLocation = localStorage.getItem(localStorgeKeyForAccountLocation);
        var activeTabTaxingDitrict = localStorage.getItem(localStorgeKeyForTaxingDistrict);
    } catch (e) {
        console.log(e);
    }
    if (activeTabEntity) {
        const triggerEl = document.querySelector(`#tabs-business-entity button[data-bs-target="${activeTabEntity}"]`);
        if (triggerEl) {
            bootstrap.Tab.getOrCreateInstance(triggerEl).show()
        }
    }
    if (activeTabLocation) {
        const triggerEl = document.querySelector(`#tabs-account-location button[data-bs-target="${activeTabLocation}"]`);
        if (triggerEl) {
            bootstrap.Tab.getOrCreateInstance(triggerEl).show()
        }
    }
    if (activeTabTaxingDitrict) {
        const triggerEl = document.querySelector(`#tabs-taxing-district button[data-bs-target="${activeTabTaxingDitrict}"]`);
        if (triggerEl) {
            bootstrap.Tab.getOrCreateInstance(triggerEl).show()
        }
    }

    var fileUploaded=urlParams.get("fileUpload");
    if(fileUploaded && activeTabLocation=="#nav-checklist")
    {
        $("#alert-file-upload-actionitem").show();
    }

    
    // remove OOTB empty subgrid heading for accessbility
    $("h3.table-info.form-subgrid-heading").each(function(){
        if($(this).text().length==0){
            $(this).remove();
        }
    });


});


function getUserTimezone() {
    // Get the user's timezone offset in minutes
    var timezoneOffset = new Date().getTimezoneOffset();

    // Convert the offset to hours and reverse the sign
    var timezoneOffsetHours = -timezoneOffset / 60;

    // Get the user's timezone name using Intl.DateTimeFormat
    var timezoneName = Intl.DateTimeFormat().resolvedOptions().timeZone;

    return {
        offset: timezoneOffsetHours,
        name: timezoneName
    };
}
function convertTimestamp(timestamp) {
    // Create a new Date object from the timestamp
    var date = new Date(timestamp);

    var userTimeZone = getUserTimezone().name;

    // Format the date and time as MM-dd-yyyy hh:mm in the specified time zone
    var options = {
        timeZone: userTimeZone,
        year: 'numeric',
        month: '2-digit',
        day: '2-digit',
        hour: '2-digit',
        minute: '2-digit',
        hour12: false
    };
    var formattedDate = date.toLocaleString('en-US', options);

    // Adjust the format to MM/dd/yyyy hh:mm
    //formattedDate = formattedDate.replace(',', '').replace(/\//g, '-');
    formattedDate = formattedDate.replace(',', '');

    return formattedDate;
}

function formatAddress(billingAddress1, billingCity, billingState, billingZip) {
    const addressParts = [];

    if (billingAddress1) {
        addressParts.push(billingAddress1);
    }
    if (billingCity) {
        addressParts.push(billingCity);
    }
    if (billingState && billingZip) {
        addressParts.push(`${billingState} ${billingZip}`);
    } else {
        if (billingState) {
            addressParts.push(billingState);
        }
        if (billingZip) {
            addressParts.push(billingZip);
        }
    }

    return addressParts.join(', ');
}
function capitalizeFirstLetter(str) {
    return str.charAt(0).toUpperCase() + str.slice(1).toLowerCase();
}

function addDollarSymbolToTextInput(field){
    $('<div class="input-group">').css({'padding-top' : '9.7px', 'padding-bottom' : '9.7px'}).appendTo(field.parent()).append('<span class="input-group-addon" style="padding-top:9.7px;padding-bottom:9.7px">$</span>').append(field);
}

function createHiddenAttribute(attributeName) {
  if($("#" + attributeName).length === 0)
  {
    $("#EntityFormView").append('<input type="hidden" id="' + attributeName + '" />');
    $("#" + attributeName).val(false);
  }
}

function getTableValues (returnHiddenBoolAttributeName, table, select, filter = "", topCount = "") {
  let annotationsApiUrl = "/_api/" + table + "?$select=" + select;
  if (filter) {
    annotationsApiUrl = annotationsApiUrl + "&$filter=" + filter;
  }  
  if (topCount) {
    annotationsApiUrl = annotationsApiUrl + "&$top=" + topCount;
  }

  webapi.safeAjax({
    type: "GET",
    url: annotationsApiUrl,
    contentType: "application/json",
    success: function(res, status, xhr) {
      if ($("#" + returnHiddenBoolAttributeName).length > 0) {
        if (res.value) {
          $("#" + returnHiddenBoolAttributeName).val(res.value.length);
        } else {
          $("#" + returnHiddenBoolAttributeName).val(0);
        }
      }
      return res.value;
    },
    error: function (xhr, textStatus, errorThrown) {
      console.log("Error while fetching " + table + ": " + errorThrown);
      $("#" + returnHiddenBoolAttributeName).val(false);
    }
  });
}

function actionItemsValidation(hiddenAttributeName, tablePluralName, selectFields, filter = "", top = "") {  
  createHiddenAttribute(hiddenAttributeName);
  getTableValues(hiddenAttributeName, tablePluralName, selectFields, filter, top);
}

function validateActionItemsStep(checklistItemsNotCompletedAttribute, uploadedDocumentsAttribute) {
  var alertelement='<div class="alert alert-danger d-flex align-items-center alert-dismissible fade show" role="alert"><div><i class="fa-solid fa-triangle-exclamation"></i>&nbsp;Please complete all required checklist items and upload necessary documents for Temporary Permit application submission.</div><button type="button" class="btn-close" data-bs-dismiss="alert" aria-label="Close"><i class="fa-solid fa-xmark"></i></button></div>';     

  if(($("#" + checklistItemsNotCompletedAttribute).val() && parseInt($("#" + checklistItemsNotCompletedAttribute).val()) > 0) ||
     ($("#" + uploadedDocumentsAttribute).val() && parseInt($("#" + uploadedDocumentsAttribute).val()) < 1)) {
    $('#Subgrid_new_5 .entity-grid').prepend(alertelement);
    return false;
  }
  else {
    return true;
  }
}